% This matlab function is designed to construct a support
% vector machine using a training dataset as input and then
% it will attempt to use this SVM to classify another
% input dataset (the test data) into a pass/fail scenario.

% Inputs:
%	x - A matrix of your xi values where each column is a 
%		different attribute and each row is a different object
%	y - The results of each test object 	

function [w,b] = nl_svm(x,y,ktype,sigma,C)

% Get the info about the x matrix
[m,n] = size(x);

% Define the K matrix that will hold 
K = zeros(m,m);

% Define each entry for the K matrix
for i=1:m
    for j=1:m
        
        % If we are using gaussian
        if(strcmp(ktype,'gauss'))
            K(i,j) = exp(-(norm(x(i,:)-x(j,:)).^2)./(2*(sigma^2)));
        elseif(strcmp(ktype,'exp'))
            K(i,j) = exp(-(norm(x(i,:)-x(j,:)))./(2*(sigma^2)));
        else
            % Otherwise we can just use a linear kernel
            K(i,j) = x(i,:)*x(j,:)' + sigma;
        end
    end
end

% Now perform the quadratic programming maximization
% Notes:
%       The 3rd and 4th inputs are of the form Ax <= b which is not a
%           constraint in our optimization
%       

% Lets try to reduce the matrix to give us a better condition number
% [U,s,V] = svd(K);
% -(s(1,1).*U(:,1)*V(:,1)')

% This formulates the equality constraint
Aeq = zeros(n,m);

for i=1:n
    Aeq(i,:) = x(:,i)'.*y';
end

beq = x\y;

K = K.*(y*y');

% Set the options we should use
opts = optimset('MaxIter',500);

%[alpha] = quadprog(-K,-y,[],[],Aeq,beq,zeros(size(y)),C.*ones(size(y)));
[alpha] = quadprog(K,y,[],[],y',0,zeros(size(y)),C.*ones(size(y)),opts);

%H = 

%[alpha] = quadprog(-K,-y,[],[],Aeq,beq,zeros(size(y)),C.*ones(size(y)));

aws = sort(alpha,'descend');
aws(1:10)

%
% Once we have the lagrange multipliers we can then formulate the w vector
% based on the lagrange multipliers
%

% Initialize w
w = zeros(n,1);

for i=1:n
    w(i) = sum(alpha.*x(:,1).*y);
end

% Then solve for b
b = mean(y - x*w);





