% This matlab script will handle some of the test cases we want to run on
% MATLAB's support vector machine capabilities
clear
clc
close all

% First read in all the data
A = dlmread('data.txt');

% The headers that are not included in the data file
headers = {'Input A','Input B','Input C','Input D','Input E','Input F','Input G'};

% seperate it into x and y
[m,n] = size(A);
x = A(:,1:n-1);
y = A(:,n);

% y data needs to be in terms of -1 or 1
y(y==0) = -1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Ok now run through the entire dataset to see which kernel function works
% best
%

% Types of kernel functions
% kernels = {'linear','quadratic','polynomial','rbf'};
% 
% % The corresponding accuracy of each kernel
% perc = zeros(length(kernels));
% 
% % Loop over the different kernels to determine their accuracy
% for i=1:length(kernels)
%     
%     
%     % Find the predicted y based on our training data
%     [yp,Q] = mat_svm(x,y,kernels{1,i},[]); 
%     
%     % Now determine how well each kernel does
%     pred = zeros(size(y));
% 
%     for j=1:size(y,1)
% 
%         % If we have a match
%         if(yp(j) == y(j))
%             pred(j) = 1;
%         else
%             pred(j) = 0;
%         end
%     end
%     
%     % Find the accuracy
%     perc(i) = 100.*sum(pred)./size(pred,1);
%     fprintf('%s\t%d\n',kernels{1,i},perc(i));
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Now that we know which kernel works best lets use this information to
% play around with some of the data


% Which kernel we are using CHANGE THIS DEPENDING ON WHAT YOU WANT TO USE
% [yp,Q] = mat_svm(x,y,'linear',[]); 
% 
% % n is the number of x parameters we have
% n = n - 1;
% 
% %x = Q.SupportVectors;
% 
% % Alright cool! Lets produce some plots now!
% for i=1:n
%     for j=1:n
%         
%         if(i == j)
%             %do nothing we dont want these plots
%         
%         else
%             
% 			% Initialize the handles to non handles
% 			H = 0;
% 			Fp = 0;
% 			Fn = 0;
% 			Tn = 0;
% 			Tp = 0;			
% 			
%             % Initialize the figure
%             figure(1)
%             hold on;
%             xlabel(headers{1,i});
%             ylabel(headers{1,j});
%             title(strcat(headers{1,i},' vs. ',headers{1,j},' Identification'));
%             
%             % Loop over all the points
%             for k=1:m
%                 
%                 % False positive
%                 if(y(k) ~= yp(k) && yp(k) > 0)
%                     Fp1 = plot(x(k,i),x(k,j),'r*','Linewidth',1.5);
%                     Fp = 1;
%                 % False negative
%                 elseif(y(k) ~= yp(k) && yp(k) < 0)
%                     Fn1 = plot(x(k,i),x(k,j),'ro','Linewidth',1.5);
%                     Fn = 1;
%                 % True negative
%                 elseif(y(k) == yp(k) && y(k) < 0)
%                     Tn1 = plot(x(k,i),x(k,j),'bo','Linewidth',1.5);
%                     Tn = 1;
%                 % True Positive
%                 elseif(y(k) == yp(k) && y(k) > 0)
%                     Tp1 = plot(x(k,i),x(k,j),'g*','Linewidth',1.5);
%                     Tp = 1;
%                 end
%             end
%             
% 			%
%             % Put the legend on it save it then kill it
% 			%
% 			
% 			% For the legend, I know this is stupidly complicated but I can think
% 			% of no better way to handle it (haha pun get it?).
% 			if(Fp && Fn && Tn && Tp)
% 				
% 				H = legend([Fp1,Fn1,Tn1,Tp1],'False Positive','False Negative', ...
% 					'True Negative','True Positive');
% 			
% 			% The most likely cases we will ever not get are false positives/negatives
% 			% if this assumption is ever incorrect... well we are screwed
% 			
% 			% Handle no false positives
% 			elseif(~Fp && Fn && Tn && Tp)			
% 			
% 				H = legend([Fn1,Tn1,Tp1],'False Negative', ...
% 					'True Negative','True Positive');
% 			
% 			% Handle no false negatives
% 			elseif(Fp && ~Fn && Tn && Tp)			
% 			
% 				H = legend([Fp1,Tn1,Tp1],'False Positive', ...
% 					'True Negative','True Positive');
% 			
% 			% Handle no false positives and negatives
% 			elseif(~Fp && ~Fn && Tn && Tp)			
% 			
% 				H = legend([Tn1,Tp1],'True Negative','True Positive');
% 				
% 			else
% 				fprintf('Error we encountered the unlikely case where there were\n')
% 				fprintf('no true positives/negatives in our plot for the case:\n');
% 				fprintf('%s vs. %s',headers{1,j},headers{1,i});
% 			end
% 			
%             hold off
%             saveas(H,strcat('twoD_',headers{1,i},'_',headers{1,j},'.png'),'png');
%             close all
%             
%             % Kill the handles
% %             delete(H);
% %             delete(Fp);
% %             delete(Fn);
% %             delete(Tn);
% %             delete(Tp);
%         end
%     end
% end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Experimenta with modified training data

% Random distortion
d = linspace(.01,1,50);


% Lets also find the time it takes for some runs
times = zeros(length(d).*5.*14);



% We want to use a variety of training points to generate some plots
for ntrain=10:10:140
    
    % Formulate the training data
    % The indices we are going to be using as training data
    itrain = round(1+(m-1)*rand(ntrain,1));

    % Choose the random data as the training data
    xtrain = x(itrain,:);
    ytrain = y(itrain,:);
    
    % Initialize percentages
	Perc = zeros(length(d),5);

% Loop over number of experiments
for k=1:5

    % Loop over the permutation sensitivity
    for i=1:length(d)

        % we are going to use the same training data but not the same test data
        xtest = x + d(i).*(2.*rand(size(x))-1);

        %Which kernel we are using CHANGE THIS DEPENDING ON WHAT YOU WANT TO USE
        tic
        [yp,Q] = mat_svm(xtrain,ytrain,'polynomial',xtest); 
        times(ntrain.*k.*i) = toc;

        % Find the percentage and record it
        Perc(i,k) = 100*(1 - (sum(.5*abs(yp-y))./m));
        
        % Store the percentage
        %Perc(i) = 100.*sum(pred)./length(pred);
        %fprintf('Percentage at %d displacement = %d\n',d(i),Perc(i));
    end

    
end
	% Since it is random we need to measure a few instances
    figure(1)
    hold on
    plot(d,Perc,'Linewidth',1.5);
    xlabel('displacement multiplier')
    ylabel('Percent Correct')
    H=title(strcat('Polynomial sensitivity for N_{train} = ',num2str(ntrain)));
    hold off
    saveas(H,strcat('poly_sens_N',num2str(ntrain),'.png'));
    close all
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This last section will choose at random a certain number of points as
%% training data and then use the rest as test data


% % Initialize the percentage matrix
% perc = zeros(50,14);
% 
% % Kernel to use
% kernel = 'rbf';
% 
% k = 1;
% 
% % Loop over all the instances we want to use
% for ntrain=10:10:140
%     
%     % Need lots of data so try 50 points
%     for i=1:50
%     
%         % The indices we are going to be using as training data
%         itrain = round(1+(m-1)*rand(ntrain,1));
% 
%         % Choose the random data as the training data
%         xtrain = x(itrain,:);
%         ytrain = y(itrain,:);
% 
%         % Run the support vector machine
%         [yp,Q] = mat_svm(xtrain,ytrain,'polynomial',x); 
% 
%         % Find the percentage and record it
%         perc(i,k) = 100*(1 - (sum(.5*abs(yp-y))./m));
%     end
%     
%     fprintf('Average percentage: %d\n',mean(perc(:,k)));
%     
%     k = k+1;
% end
% 
% 
% % Now that we have all the data plot it
% figure(1)
% hold on
% boxplot(perc,(10:10:140));
% xlabel('Number of training points used');
% ylabel('Percentage');
% title('Percent correct for Polynomial kernel');
% hold off

        


    
    




